#!/usr/bin/env python3
"""
VigileGuard Setup Script
Simplified setup.py that defers to pyproject.toml
"""

from setuptools import setup

# All configuration is now in pyproject.toml
# This setup.py is kept for compatibility only
setup()